@@ -46,7 +46,6 @@ enum connman_element_type {
 	CONNMAN_ELEMENT_TYPE_DEVICE     = 3,
 	CONNMAN_ELEMENT_TYPE_NETWORK    = 4,
 	CONNMAN_ELEMENT_TYPE_SERVICE    = 5,
-	CONNMAN_ELEMENT_TYPE_PPP        = 6,
 	CONNMAN_ELEMENT_TYPE_IPV4       = 7,
 	CONNMAN_ELEMENT_TYPE_IPV6       = 8,
 	CONNMAN_ELEMENT_TYPE_DHCP       = 9,
@@ -107,6 +106,7 @@ struct connman_element {
 		gchar *network;
 		gchar *broadcast;
 		gchar *nameserver;
+		gchar *timeserver;
 	} ipv4;
 };
 
@@ -121,6 +121,10 @@ int connman_element_set_string(struct connman_element *element,
 					const char *key, const char *value);
 const char *connman_element_get_string(struct connman_element *element,
 							const char *key);
+int connman_element_set_bool(struct connman_element *element,
+				const char *key, connman_bool_t value);
+connman_bool_t connman_element_get_bool(struct connman_element *element,
+							const char *key);
 int connman_element_set_uint8(struct connman_element *element,
 				const char *key, connman_uint8_t value);
 connman_uint8_t connman_element_get_uint8(struct connman_element *element,
