@@ -36,15 +36,14 @@ struct connman_device {
 	struct connman_element element;
 	enum connman_device_type type;
 	enum connman_device_mode mode;
-	connman_bool_t secondary;
 	connman_bool_t offlinemode;
 	connman_bool_t blocked;
 	connman_bool_t powered;
 	connman_bool_t powered_pending;
 	connman_bool_t powered_persistent;
-	connman_bool_t carrier;
 	connman_bool_t scanning;
 	connman_bool_t disconnected;
+	connman_bool_t reconnect;
 	connman_uint16_t scan_interval;
 	char *name;
 	char *node;
@@ -55,7 +54,6 @@ struct connman_device {
 	int phyindex;
 	unsigned int connections;
 	guint scan_timeout;
-	struct connman_ipconfig *ipconfig;
 
 	struct connman_device_driver *driver;
 	void *driver_data;
@@ -114,6 +112,11 @@ static void force_scan_trigger(struct connman_device *device)
 					device_scan_trigger, device);
 }
 
+void connman_device_schedule_scan(struct connman_device *device)
+{
+	reset_scan_trigger(device);
+}
+
 static const char *type2description(enum connman_device_type type)
 {
 	switch (type) {
@@ -130,11 +133,9 @@ static const char *type2description(enum connman_device_type type)
 		return "Bluetooth";
 	case CONNMAN_DEVICE_TYPE_GPS:
 		return "GPS";
+	case CONNMAN_DEVICE_TYPE_CELLULAR:
 	case CONNMAN_DEVICE_TYPE_MBM:
 	case CONNMAN_DEVICE_TYPE_HSO:
-	case CONNMAN_DEVICE_TYPE_NOZOMI:
-	case CONNMAN_DEVICE_TYPE_HUAWEI:
-	case CONNMAN_DEVICE_TYPE_NOVATEL:
 		return "Cellular";
 	}
 
@@ -157,11 +158,9 @@ static const char *type2string(enum connman_device_type type)
 		return "bluetooth";
 	case CONNMAN_DEVICE_TYPE_GPS:
 		return "gps";
+	case CONNMAN_DEVICE_TYPE_CELLULAR:
 	case CONNMAN_DEVICE_TYPE_MBM:
 	case CONNMAN_DEVICE_TYPE_HSO:
-	case CONNMAN_DEVICE_TYPE_HUAWEI:
-	case CONNMAN_DEVICE_TYPE_NOZOMI:
-	case CONNMAN_DEVICE_TYPE_NOVATEL:
 		return "cellular";
 	}
 
@@ -176,9 +175,6 @@ enum connman_service_type __connman_device_get_service_type(struct connman_devic
 	case CONNMAN_DEVICE_TYPE_UNKNOWN:
 	case CONNMAN_DEVICE_TYPE_VENDOR:
 	case CONNMAN_DEVICE_TYPE_GPS:
-	case CONNMAN_DEVICE_TYPE_NOZOMI:
-	case CONNMAN_DEVICE_TYPE_HUAWEI:
-	case CONNMAN_DEVICE_TYPE_NOVATEL:
 		break;
 	case CONNMAN_DEVICE_TYPE_ETHERNET:
 		return CONNMAN_SERVICE_TYPE_ETHERNET;
@@ -188,6 +184,7 @@ enum connman_service_type __connman_device_get_service_type(struct connman_devic
 		return CONNMAN_SERVICE_TYPE_WIMAX;
 	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
 		return CONNMAN_SERVICE_TYPE_BLUETOOTH;
+	case CONNMAN_DEVICE_TYPE_CELLULAR:
 	case CONNMAN_DEVICE_TYPE_MBM:
 	case CONNMAN_DEVICE_TYPE_HSO:
 		return CONNMAN_SERVICE_TYPE_CELLULAR;
@@ -196,105 +193,11 @@ enum connman_service_type __connman_device_get_service_type(struct connman_devic
 	return CONNMAN_SERVICE_TYPE_UNKNOWN;
 }
 
-static int set_connected(struct connman_device *device,
-						connman_bool_t connected)
+static void powered_changed(struct connman_device *device)
 {
-	if (connected == TRUE) {
-		enum connman_element_type type = CONNMAN_ELEMENT_TYPE_UNKNOWN;
-		struct connman_element *element;
-
-		device->disconnected = TRUE;
-
-		switch (device->element.ipv4.method) {
-		case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
-		case CONNMAN_IPCONFIG_METHOD_IGNORE:
-			return 0;
-		case CONNMAN_IPCONFIG_METHOD_STATIC:
-			type = CONNMAN_ELEMENT_TYPE_IPV4;
-			break;
-		case CONNMAN_IPCONFIG_METHOD_DHCP:
-			type = CONNMAN_ELEMENT_TYPE_DHCP;
-			break;
-		}
-
-		element = connman_element_create(NULL);
-		if (element != NULL) {
-			struct connman_service *service;
-
-			element->type  = type;
-			element->index = device->element.index;
-
-			if (connman_element_register(element,
-							&device->element) < 0)
-				connman_element_unref(element);
-
-			device->disconnected = FALSE;
-
-			service = __connman_service_lookup_from_device(device);
-			__connman_service_indicate_state(service,
-					CONNMAN_SERVICE_STATE_CONFIGURATION);
-		}
-	} else {
-		struct connman_service *service;
-
-		connman_element_unregister_children(&device->element);
-
-		device->disconnected = TRUE;
-
-		service = __connman_service_lookup_from_device(device);
-		__connman_service_indicate_state(service,
-					CONNMAN_SERVICE_STATE_IDLE);
-	}
-
-	if (connected == TRUE) {
-		enum connman_service_type type;
-
-		type = __connman_device_get_service_type(device);
-		__connman_notifier_connect(type);
-	} else {
-		enum connman_service_type type;
-
-		type = __connman_device_get_service_type(device);
-		__connman_notifier_disconnect(type);
-	}
-
-	return 0;
-}
-
-static int set_carrier(struct connman_device *device, connman_bool_t carrier)
-{
-	if (carrier == TRUE)
-		__connman_profile_add_device(device);
-	else
-		__connman_profile_remove_device(device);
-
-	return set_connected(device, carrier);
-}
-
-static int powered_changed(struct connman_device *device)
-{
-	DBusMessage *signal;
-	DBusMessageIter entry, value;
-	const char *key = "Powered";
-
-	signal = dbus_message_new_signal(device->element.path,
-				CONNMAN_DEVICE_INTERFACE, "PropertyChanged");
-	if (signal == NULL)
-		return -ENOMEM;
-
-	dbus_message_iter_init_append(signal, &entry);
-
-	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
-
-	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
-					DBUS_TYPE_BOOLEAN_AS_STRING, &value);
-	dbus_message_iter_append_basic(&value, DBUS_TYPE_BOOLEAN,
-							&device->powered);
-	dbus_message_iter_close_container(&entry, &value);
-
-	g_dbus_send_message(connection, signal);
-
-	return 0;
+	connman_dbus_property_changed_basic(device->element.path,
+				CONNMAN_DEVICE_INTERFACE, "Powered",
+					DBUS_TYPE_BOOLEAN, &device->powered);
 }
 
 static int set_powered(struct connman_device *device, connman_bool_t powered)
@@ -325,12 +228,12 @@ static int set_powered(struct connman_device *device, connman_bool_t powered)
 	} else {
 		device->powered_pending = powered;
 
+		device->reconnect = FALSE;
+
 		clear_scan_trigger(device);
 
 		g_hash_table_remove_all(device->networks);
 
-		set_carrier(device, FALSE);
-
 		if (driver->disable) {
 			err = driver->disable(device);
 			if (err == 0)
@@ -349,6 +252,11 @@ static int set_powered(struct connman_device *device, connman_bool_t powered)
 	return err;
 }
 
+void __connman_device_list(DBusMessageIter *iter, void *user_data)
+{
+	__connman_element_list(NULL, CONNMAN_ELEMENT_TYPE_DEVICE, iter);
+}
+
 static void append_path(gpointer key, gpointer value, gpointer user_data)
 {
 	struct connman_element *element = value;
@@ -358,24 +266,11 @@ static void append_path(gpointer key, gpointer value, gpointer user_data)
 							&element->path);
 }
 
-static void append_networks(struct connman_device *device,
-						DBusMessageIter *entry)
+static void append_networks(DBusMessageIter *iter, void *user_data)
 {
-	DBusMessageIter value, iter;
-	const char *key = "Networks";
+	struct connman_device *device = user_data;
 
-	dbus_message_iter_append_basic(entry, DBUS_TYPE_STRING, &key);
-
-	dbus_message_iter_open_container(entry, DBUS_TYPE_VARIANT,
-		DBUS_TYPE_ARRAY_AS_STRING DBUS_TYPE_OBJECT_PATH_AS_STRING,
-								&value);
-
-	dbus_message_iter_open_container(&value, DBUS_TYPE_ARRAY,
-				DBUS_TYPE_OBJECT_PATH_AS_STRING, &iter);
-	g_hash_table_foreach(device->networks, append_path, &iter);
-	dbus_message_iter_close_container(&value, &iter);
-
-	dbus_message_iter_close_container(entry, &value);
+	g_hash_table_foreach(device->networks, append_path, iter);
 }
 
 static DBusMessage *get_properties(DBusConnection *conn,
@@ -383,7 +278,7 @@ static DBusMessage *get_properties(DBusConnection *conn,
 {
 	struct connman_device *device = data;
 	DBusMessage *reply;
-	DBusMessageIter array, dict, entry;
+	DBusMessageIter array, dict;
 	const char *str;
 
 	DBG("conn %p", conn);
@@ -398,55 +293,47 @@ static DBusMessage *get_properties(DBusConnection *conn,
 
 	dbus_message_iter_init_append(reply, &array);
 
-	dbus_message_iter_open_container(&array, DBUS_TYPE_ARRAY,
-			DBUS_DICT_ENTRY_BEGIN_CHAR_AS_STRING
-			DBUS_TYPE_STRING_AS_STRING DBUS_TYPE_VARIANT_AS_STRING
-			DBUS_DICT_ENTRY_END_CHAR_AS_STRING, &dict);
+	connman_dbus_dict_open(&array, &dict);
 
 	if (device->name != NULL)
-		connman_dbus_dict_append_variant(&dict, "Name",
+		connman_dbus_dict_append_basic(&dict, "Name",
 					DBUS_TYPE_STRING, &device->name);
 
 	str = type2string(device->type);
 	if (str != NULL)
-		connman_dbus_dict_append_variant(&dict, "Type",
+		connman_dbus_dict_append_basic(&dict, "Type",
 						DBUS_TYPE_STRING, &str);
 
 	if (device->address != NULL)
-		connman_dbus_dict_append_variant(&dict, "Address",
+		connman_dbus_dict_append_basic(&dict, "Address",
 					DBUS_TYPE_STRING, &device->address);
 
 	if (device->interface != NULL)
-		connman_dbus_dict_append_variant(&dict, "Interface",
+		connman_dbus_dict_append_basic(&dict, "Interface",
 					DBUS_TYPE_STRING, &device->interface);
 
-	connman_dbus_dict_append_variant(&dict, "Powered",
+	connman_dbus_dict_append_basic(&dict, "Powered",
 					DBUS_TYPE_BOOLEAN, &device->powered);
 
 	if (device->driver && device->driver->scan)
-		connman_dbus_dict_append_variant(&dict, "Scanning",
+		connman_dbus_dict_append_basic(&dict, "Scanning",
 					DBUS_TYPE_BOOLEAN, &device->scanning);
 
 	switch (device->mode) {
 	case CONNMAN_DEVICE_MODE_UNKNOWN:
 		break;
-	case CONNMAN_DEVICE_MODE_TRANSPORT_IP:
-		__connman_element_append_ipv4(&device->element, &dict);
-		break;
 	case CONNMAN_DEVICE_MODE_NETWORK_SINGLE:
 	case CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE:
 		if (device->scan_interval > 0)
-			connman_dbus_dict_append_variant(&dict, "ScanInterval",
+			connman_dbus_dict_append_basic(&dict, "ScanInterval",
 				DBUS_TYPE_UINT16, &device->scan_interval);
 
-		dbus_message_iter_open_container(&dict, DBUS_TYPE_DICT_ENTRY,
-								NULL, &entry);
-		append_networks(device, &entry);
-		dbus_message_iter_close_container(&dict, &entry);
+		connman_dbus_dict_append_array(&dict, "Networks",
+				DBUS_TYPE_OBJECT_PATH, append_networks, device);
 		break;
 	}
 
-	dbus_message_iter_close_container(&array, &dict);
+	connman_dbus_dict_close(&array, &dict);
 
 	return reply;
 }
@@ -532,8 +419,6 @@ static DBusMessage *set_property(DBusConnection *conn,
 
 		switch (device->mode) {
 		case CONNMAN_DEVICE_MODE_UNKNOWN:
-		case CONNMAN_DEVICE_MODE_TRANSPORT_IP:
-			return __connman_error_invalid_arguments(msg);
 		case CONNMAN_DEVICE_MODE_NETWORK_SINGLE:
 		case CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE:
 			break;
@@ -551,25 +436,6 @@ static DBusMessage *set_property(DBusConnection *conn,
 
 			reset_scan_trigger(device);
 		}
-	} else if (g_str_has_prefix(name, "IPv4.") == TRUE) {
-		int err;
-
-		if (device->ipconfig == NULL)
-			return __connman_error_invalid_property(msg);
-
-		switch (device->mode) {
-		case CONNMAN_DEVICE_MODE_UNKNOWN:
-		case CONNMAN_DEVICE_MODE_NETWORK_SINGLE:
-		case CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE:
-			return __connman_error_invalid_arguments(msg);
-		case CONNMAN_DEVICE_MODE_TRANSPORT_IP:
-			break;
-		}
-
-		err = __connman_ipconfig_set_ipv4(device->ipconfig,
-							name + 5, &value);
-		if (err < 0)
-			return __connman_error_failed(msg, -err);
 	} else
 		return __connman_error_invalid_property(msg);
 
@@ -586,7 +452,6 @@ static DBusMessage *propose_scan(DBusConnection *conn,
 
 	switch (device->mode) {
 	case CONNMAN_DEVICE_MODE_UNKNOWN:
-	case CONNMAN_DEVICE_MODE_TRANSPORT_IP:
 		return __connman_error_not_supported(msg);
 	case CONNMAN_DEVICE_MODE_NETWORK_SINGLE:
 	case CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE:
@@ -613,40 +478,11 @@ static GDBusSignalTable device_signals[] = {
 	{ },
 };
 
-static void append_devices(DBusMessageIter *entry)
-{
-	DBusMessageIter value, iter;
-	const char *key = "Devices";
-
-	dbus_message_iter_append_basic(entry, DBUS_TYPE_STRING, &key);
-
-	dbus_message_iter_open_container(entry, DBUS_TYPE_VARIANT,
-		DBUS_TYPE_ARRAY_AS_STRING DBUS_TYPE_OBJECT_PATH_AS_STRING,
-								&value);
-
-	dbus_message_iter_open_container(&value, DBUS_TYPE_ARRAY,
-				DBUS_TYPE_OBJECT_PATH_AS_STRING, &iter);
-	__connman_element_list(NULL, CONNMAN_ELEMENT_TYPE_DEVICE, &iter);
-	dbus_message_iter_close_container(&value, &iter);
-
-	dbus_message_iter_close_container(entry, &value);
-}
-
 static void emit_devices_signal(void)
 {
-	DBusMessage *signal;
-	DBusMessageIter entry;
-
-	signal = dbus_message_new_signal(CONNMAN_MANAGER_PATH,
-				CONNMAN_MANAGER_INTERFACE, "PropertyChanged");
-	if (signal == NULL)
-		return;
-
-	dbus_message_iter_init_append(signal, &entry);
-
-	append_devices(&entry);
-
-	g_dbus_send_message(connection, signal);
+	connman_dbus_property_changed_array(CONNMAN_MANAGER_PATH,
+			CONNMAN_MANAGER_INTERFACE, "Devices",
+			DBUS_TYPE_OBJECT_PATH, __connman_device_list, NULL);
 }
 
 static int register_interface(struct connman_element *element)
@@ -707,10 +543,6 @@ static int setup_device(struct connman_device *device)
 	case CONNMAN_DEVICE_MODE_NETWORK_SINGLE:
 	case CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE:
 		break;
-	case CONNMAN_DEVICE_MODE_TRANSPORT_IP:
-		if (device->carrier == TRUE && device->secondary == FALSE)
-			__connman_profile_add_device(device);
-		break;
 	}
 
 	if (device->offlinemode == FALSE &&
@@ -756,10 +588,6 @@ static void remove_device(struct connman_device *device)
 	case CONNMAN_DEVICE_MODE_NETWORK_SINGLE:
 	case CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE:
 		break;
-	case CONNMAN_DEVICE_MODE_TRANSPORT_IP:
-		if (device->secondary == FALSE)
-			__connman_profile_remove_device(device);
-		break;
 	}
 
 	type = __connman_device_get_service_type(device);
@@ -863,6 +691,8 @@ static void device_destruct(struct connman_element *element)
 		device->timeout = 0;
 	}
 
+	clear_scan_trigger(device);
+
 	if (device->pending != NULL) {
 		dbus_message_unref(device->pending);
 		device->pending = NULL;
@@ -875,11 +705,6 @@ static void device_destruct(struct connman_element *element)
 	g_free(device->control);
 	g_free(device->interface);
 
-	if (device->ipconfig != NULL) {
-		connman_ipconfig_unref(device->ipconfig);
-		device->ipconfig = NULL;
-	}
-
 	g_free(device->last_network);
 
 	g_hash_table_destroy(device->networks);
@@ -924,10 +749,9 @@ struct connman_device *connman_device_create(const char *node,
 
 	device->element.ipv4.method = CONNMAN_IPCONFIG_METHOD_DHCP;
 
-	device->type      = type;
-	device->name      = g_strdup(type2description(device->type));
-	device->mode      = CONNMAN_DEVICE_MODE_UNKNOWN;
-	device->secondary = FALSE;
+	device->type = type;
+	device->name = g_strdup(type2description(device->type));
+	device->mode = CONNMAN_DEVICE_MODE_UNKNOWN;
 
 	device->powered_persistent = TRUE;
 
@@ -951,11 +775,9 @@ struct connman_device *connman_device_create(const char *node,
 	case CONNMAN_DEVICE_TYPE_GPS:
 		device->scan_interval = 0;
 		break;
+	case CONNMAN_DEVICE_TYPE_CELLULAR:
 	case CONNMAN_DEVICE_TYPE_MBM:
 	case CONNMAN_DEVICE_TYPE_HSO:
-	case CONNMAN_DEVICE_TYPE_NOZOMI:
-	case CONNMAN_DEVICE_TYPE_HUAWEI:
-	case CONNMAN_DEVICE_TYPE_NOVATEL:
 		device->scan_interval = 0;
 		break;
 	}
@@ -1039,15 +861,6 @@ const char *connman_device_get_path(struct connman_device *device)
 void connman_device_set_index(struct connman_device *device, int index)
 {
 	device->element.index = index;
-
-	if (device->ipconfig != NULL) {
-		if (index == connman_ipconfig_get_index(device->ipconfig))
-			return;
-
-		connman_ipconfig_unref(device->ipconfig);
-	}
-
-	device->ipconfig = connman_ipconfig_create(index);
 }
 
 /**
@@ -1148,30 +961,6 @@ enum connman_device_mode connman_device_get_mode(struct connman_device *device)
 	return device->mode;
 }
 
-/**
- * connman_device_set_secondary:
- * @device: device structure
- * @secondary: secondary value
- *
- * Change secondary value of device
- */
-void connman_device_set_secondary(struct connman_device *device,
-						connman_bool_t secondary)
-{
-	device->secondary = secondary;
-}
-
-/**
- * connman_device_get_secondary:
- * @device: device structure
- *
- * Get secondary value of device
- */
-connman_bool_t connman_device_get_secondary(struct connman_device *device)
-{
-	return device->secondary;
-}
-
 /**
  * connman_device_set_powered:
  * @device: device structure
@@ -1250,35 +1039,6 @@ int __connman_device_set_blocked(struct connman_device *device,
 	return set_powered(device, powered);
 }
 
-/**
- * connman_device_set_carrier:
- * @device: device structure
- * @carrier: carrier state
- *
- * Change carrier state of device (only for device without scanning)
- */
-int connman_device_set_carrier(struct connman_device *device,
-						connman_bool_t carrier)
-{
-	DBG("device %p carrier %d", device, carrier);
-
-	switch (device->mode) {
-	case CONNMAN_DEVICE_MODE_UNKNOWN:
-	case CONNMAN_DEVICE_MODE_NETWORK_SINGLE:
-	case CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE:
-		return -EINVAL;
-	case CONNMAN_DEVICE_MODE_TRANSPORT_IP:
-		break;
-	}
-
-	if (device->carrier == carrier)
-		return -EALREADY;
-
-	device->carrier = carrier;
-
-	return set_carrier(device, device->carrier);
-}
-
 int __connman_device_scan(struct connman_device *device)
 {
 	if (!device->driver || !device->driver->scan)
@@ -1348,6 +1108,8 @@ int __connman_device_disable(struct connman_device *device)
 
 	device->powered_pending = FALSE;
 
+	device->reconnect = FALSE;
+
 	clear_scan_trigger(device);
 
 	g_hash_table_remove_all(device->networks);
@@ -1375,19 +1137,6 @@ int __connman_device_disable_persistent(struct connman_device *device)
 	return __connman_device_disable(device);
 }
 
-int __connman_device_connect(struct connman_device *device)
-{
-	DBG("device %p", device);
-
-	if (device->disconnected == FALSE)
-		return -EINVAL;
-
-	if (device->driver && device->driver->connect)
-		device->driver->connect(device);
-
-	return 0;
-}
-
 int __connman_device_disconnect(struct connman_device *device)
 {
 	GHashTableIter iter;
@@ -1402,12 +1151,25 @@ int __connman_device_disconnect(struct connman_device *device)
 	while (g_hash_table_iter_next(&iter, &key, &value) == TRUE) {
 		struct connman_network *network = value;
 
+		if (__connman_network_get_connecting(network) == TRUE) {
+			/*
+			 * Skip network in the process of connecting.
+			 * This is a workaround for WiFi networks serviced
+			 * by the supplicant plugin that hold a reference
+			 * to the network.  If we disconnect the network
+			 * here then the referenced object will not be
+			 * registered and usage (like launching DHCP client)
+			 * will fail.  There is nothing to be gained by
+			 * removing the network here anyway.
+			 */
+			connman_warn("Skipping disconnect of %s",
+				connman_network_get_identifier(network));
+			continue;
+		}
+
 		__connman_network_disconnect(network);
 	}
 
-	if (device->driver && device->driver->disconnect)
-		device->driver->disconnect(device);
-
 	return 0;
 }
 
@@ -1444,26 +1206,9 @@ void __connman_device_cleanup_networks(struct connman_device *device)
 
 static void scanning_changed(struct connman_device *device)
 {
-	DBusMessage *signal;
-	DBusMessageIter entry, value;
-	const char *key = "Scanning";
-
-	signal = dbus_message_new_signal(device->element.path,
-				CONNMAN_DEVICE_INTERFACE, "PropertyChanged");
-	if (signal == NULL)
-		return;
-
-	dbus_message_iter_init_append(signal, &entry);
-
-	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
-
-	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
-					DBUS_TYPE_BOOLEAN_AS_STRING, &value);
-	dbus_message_iter_append_basic(&value, DBUS_TYPE_BOOLEAN,
-							&device->scanning);
-	dbus_message_iter_close_container(&entry, &value);
-
-	g_dbus_send_message(connection, signal);
+	connman_dbus_property_changed_basic(device->element.path,
+				CONNMAN_DEVICE_INTERFACE, "Scanning",
+					DBUS_TYPE_BOOLEAN, &device->scanning);
 }
 
 static void mark_network_available(gpointer key, gpointer value,
@@ -1542,7 +1287,6 @@ int connman_device_set_disconnected(struct connman_device *device,
 
 	switch (device->mode) {
 	case CONNMAN_DEVICE_MODE_UNKNOWN:
-	case CONNMAN_DEVICE_MODE_TRANSPORT_IP:
 		return -EINVAL;
 	case CONNMAN_DEVICE_MODE_NETWORK_SINGLE:
 	case CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE:
@@ -1571,33 +1315,6 @@ connman_bool_t connman_device_get_disconnected(struct connman_device *device)
 	return device->disconnected;
 }
 
-/**
- * connman_device_set_connected:
- * @device: device structure
- * @connected: connected state
- *
- * Change connected state of device (for Ethernet like devices)
- */
-int connman_device_set_connected(struct connman_device *device,
-						connman_bool_t connected)
-{
-	DBG("device %p connected %d", device, connected);
-
-	switch (device->mode) {
-	case CONNMAN_DEVICE_MODE_UNKNOWN:
-	case CONNMAN_DEVICE_MODE_NETWORK_SINGLE:
-	case CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE:
-		return -EINVAL;
-	case CONNMAN_DEVICE_MODE_TRANSPORT_IP:
-		break;
-	}
-
-	if (device->carrier == FALSE)
-		return -ENOTCONN;
-
-	return set_connected(device, connected);
-}
-
 /**
  * connman_device_set_string:
  * @device: device structure
@@ -1708,9 +1425,11 @@ int connman_device_add_network(struct connman_device *device,
 
 	DBG("device %p network %p", device, network);
 
+	if (identifier == NULL)
+		return -EINVAL;
+
 	switch (device->mode) {
 	case CONNMAN_DEVICE_MODE_UNKNOWN:
-	case CONNMAN_DEVICE_MODE_TRANSPORT_IP:
 		return -EINVAL;
 	case CONNMAN_DEVICE_MODE_NETWORK_SINGLE:
 	case CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE:
@@ -1764,6 +1483,11 @@ int connman_device_remove_network(struct connman_device *device,
 	return 0;
 }
 
+void connman_device_remove_all_networks(struct connman_device *device)
+{
+	g_hash_table_remove_all(device->networks);
+}
+
 void __connman_device_set_network(struct connman_device *device,
 					struct connman_network *network)
 {
@@ -1790,6 +1514,18 @@ void __connman_device_set_network(struct connman_device *device,
 	}
 }
 
+void __connman_device_set_reconnect(struct connman_device *device,
+						connman_bool_t reconnect)
+{
+	device->reconnect = reconnect;
+}
+
+connman_bool_t  __connman_device_get_reconnect(
+				struct connman_device *device)
+{
+	return device->reconnect;
+}
+
 /**
  * connman_device_register:
  * @device: device structure
@@ -1918,7 +1654,7 @@ static int device_load(struct connman_device *device)
 
 	DBG("device %p", device);
 
-	keyfile = __connman_storage_open(ident);
+	keyfile = __connman_storage_open_profile(ident);
 	if (keyfile == NULL)
 		return 0;
 
@@ -1934,7 +1670,6 @@ static int device_load(struct connman_device *device)
 
 	switch (device->mode) {
 	case CONNMAN_DEVICE_MODE_UNKNOWN:
-	case CONNMAN_DEVICE_MODE_TRANSPORT_IP:
 		break;
 	case CONNMAN_DEVICE_MODE_NETWORK_SINGLE:
 	case CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE:
@@ -1949,7 +1684,7 @@ static int device_load(struct connman_device *device)
 done:
 	g_free(identifier);
 
-	__connman_storage_close(ident, keyfile, FALSE);
+	__connman_storage_close_profile(ident, keyfile, FALSE);
 
 	return 0;
 }
@@ -1962,7 +1697,7 @@ static int device_save(struct connman_device *device)
 
 	DBG("device %p", device);
 
-	keyfile = __connman_storage_open(ident);
+	keyfile = __connman_storage_open_profile(ident);
 	if (keyfile == NULL)
 		return 0;
 
@@ -1975,7 +1710,6 @@ static int device_save(struct connman_device *device)
 
 	switch (device->mode) {
 	case CONNMAN_DEVICE_MODE_UNKNOWN:
-	case CONNMAN_DEVICE_MODE_TRANSPORT_IP:
 		break;
 	case CONNMAN_DEVICE_MODE_NETWORK_SINGLE:
 	case CONNMAN_DEVICE_MODE_NETWORK_MULTIPLE:
@@ -1988,7 +1722,7 @@ static int device_save(struct connman_device *device)
 done:
 	g_free(identifier);
 
-	__connman_storage_close(ident, keyfile, TRUE);
+	__connman_storage_close_profile(ident, keyfile, TRUE);
 
 	return 0;
 }
