@@ -27,29 +27,12 @@
 #include <stdlib.h>
 #include <sys/types.h>
 
-#define LIBUDEV_I_KNOW_THE_API_IS_SUBJECT_TO_CHANGE
 #include <libudev.h>
 
 #include <glib.h>
 
 #include "connman.h"
 
-#ifdef NEED_UDEV_MONITOR_FILTER
-static int udev_monitor_filter_add_match_subsystem_devtype(struct udev_monitor *udev_monitor,
-				const char *subsystem, const char *devtype)
-{
-	return -EINVAL;
-}
-static int udev_monitor_filter_update(struct udev_monitor *udev_monitor)
-{
-	return -EINVAL;
-}
-static int udev_monitor_filter_remove(struct udev_monitor *udev_monitor)
-{
-	return -EINVAL;
-}
-#endif
-
 static gboolean rfkill_processing = FALSE;
 
 static GSList *device_list = NULL;
@@ -108,11 +91,8 @@ static void add_net_device(struct udev_device *udev_device)
 	case CONNMAN_DEVICE_TYPE_VENDOR:
 	case CONNMAN_DEVICE_TYPE_WIMAX:
 	case CONNMAN_DEVICE_TYPE_BLUETOOTH:
-	case CONNMAN_DEVICE_TYPE_GPS:
-	case CONNMAN_DEVICE_TYPE_NOZOMI:
-	case CONNMAN_DEVICE_TYPE_HUAWEI:
-	case CONNMAN_DEVICE_TYPE_NOVATEL:
 	case CONNMAN_DEVICE_TYPE_CELLULAR:
+	case CONNMAN_DEVICE_TYPE_GPS:
 		return;
 	case CONNMAN_DEVICE_TYPE_ETHERNET:
 	case CONNMAN_DEVICE_TYPE_WIFI:
@@ -257,7 +237,7 @@ static void print_properties(struct udev_device *device, const char *prefix)
 				g_str_equal(name, "IFINDEX") == TRUE ||
 				g_str_equal(name, "DEVNAME") == TRUE ||
 				g_str_equal(name, "DEVPATH") == TRUE)
-			connman_debug("%s%s = %s", prefix, name, value);
+			DBG("%s%s = %s", prefix, name, value);
 
 		entry = udev_list_entry_get_next(entry);
 	}
@@ -268,7 +248,7 @@ static void print_device(struct udev_device *device, const char *action)
 	const char *subsystem, *sysname, *driver, *devtype = NULL;
 	struct udev_device *parent;
 
-	connman_debug("=== %s ===", action);
+	DBG("=== %s ===", action);
 	print_properties(device, "");
 
 	parent = udev_device_get_parent(device);
